## $Id: tests.py 13381 2015-11-02 13:42:10Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import os
from datetime import datetime, date, timedelta
from zope.component import createObject, getUtility
from zope.catalog.interfaces import ICatalog
from hurry.workflow.interfaces import IWorkflowState
from waeup.kofa.students.tests.test_browser import StudentsFullSetup
from waeup.kofa.applicants.tests.test_browser import ApplicantsFullSetup
from waeup.kofa.configuration import SessionConfiguration
from waeup.aaue.testing import FunctionalLayer

# Also run tests that send requests to external servers?
#   If you enable this, please make sure the external services
#   do exist really and are not bothered by being spammed by a test programme.
EXTERNAL_TESTS = False

def external_test(func):
    if not EXTERNAL_TESTS:
        myself = __file__
        if myself.endswith('.pyc'):
            myself = myself[:-2]
        print "WARNING: external tests are skipped!"
        print "WARNING: edit %s to enable them." % myself
        return
    return func


class InterswitchTestsApplicants(ApplicantsFullSetup):
    """Tests for the Interswitch payment gateway.
    """

    layer = FunctionalLayer

    def setUp(self):
        super(InterswitchTestsApplicants, self).setUp()
        configuration = SessionConfiguration()
        configuration.academic_session = datetime.now().year - 2
        self.app['configuration'].addSessionConfiguration(configuration)
        self.configuration = configuration
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.manage_path)
        #IWorkflowState(self.student).setState('started')
        super(InterswitchTestsApplicants, self).fill_correct_values()
        self.browser.getControl(name="form.programme_type").value = ['regular']
        self.applicantscontainer.application_fee = 1000.0
        self.browser.getControl(name="form.nationality").value = ['NG']
        self.browser.getControl(name="transition").value = ['start']
        self.browser.getControl("Save").click()
        self.browser.getControl("Add online").click()
        self.assertTrue('ticket created' in self.browser.contents)
        #self.browser.getLink(value).click()
        self.assertTrue('Amount Authorized' in self.browser.contents)
        self.assertTrue('<span>1000.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url

    def test_interswitch_form(self):
        # Manager can access InterswitchForm
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('Total Amount Authorized:' in self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="100000" />'
            in self.browser.contents)
        delta = timedelta(days=8)
        self.applicant.values()[0].creation_date -= delta
        self.browser.open(self.payment_url)
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue(
            'This payment ticket is too old. Please create a new ticket'
            in self.browser.contents)

    @external_test
    def test_webservice(self):

        self.browser.open(self.payment_url + '/request_webservice')
        self.assertTrue('Unsuccessful callback' in self.browser.contents)
        # The payment is now in state failed
        self.assertTrue('<span>Failed</span>' in self.browser.contents)

class InterswitchTestsStudents(StudentsFullSetup):
    """Tests for the Interswitch payment gateway.
    """

    layer = FunctionalLayer

    def setUp(self):
        super(InterswitchTestsStudents, self).setUp()

    def test_interswitch_form_pt_school_fees(self):
        self.certificate.study_mode = 'ug_pt'
        self.student['studycourse'].entry_session = 2013
        self.student['studycourse'].certificate.school_fee_1 = 51750.0
        # Manager can access InterswitchForm
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('cleared')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['schoolfee']
        self.browser.getControl("Create ticket").click()
        self.assertTrue('ticket created' in self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertTrue('Amount Authorized' in self.browser.contents)
        self.assertTrue('<span>51750.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue(
            'Please pay acceptance fee first' in self.browser.contents)
        acc_payment = createObject('waeup.StudentOnlinePayment')
        acc_payment.p_state = 'paid'
        acc_payment.p_category = 'clearance'
        self.student['payments']['xyz'] = acc_payment
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="105" />'
                        in self.browser.contents)
        self.assertTrue('Total Amount Authorized:' in self.browser.contents)
        self.assertEqual(self.student.current_mode, 'ug_pt')
        self.assertTrue(
            '<input type="hidden" name="amount" value="5175000" />'
            in self.browser.contents)
        self.assertTrue(
            'item_name="School Fee" item_amt="4600000" bank_id="7" '
            'acct_num="1014847058"' in self.browser.contents)

    def test_interswitch_form_regular_school_fees(self):
        self.certificate.study_mode = 'ug_ft'
        self.student['studycourse'].entry_session = 2013
        self.student['studycourse'].certificate.school_fee_1 = 51750.0
        # Manager can access InterswitchForm
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('cleared')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['schoolfee']
        self.browser.getControl("Create ticket").click()
        self.assertTrue('ticket created' in
                           self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertTrue('Amount Authorized' in
                           self.browser.contents)
        self.assertTrue('<span>51750.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue(
            'Please pay acceptance fee first' in self.browser.contents)
        acc_payment = createObject('waeup.StudentOnlinePayment')
        acc_payment.p_state = 'paid'
        acc_payment.p_category = 'clearance'
        self.student['payments']['xyz'] = acc_payment
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="101" />'
                        in self.browser.contents)
        self.assertTrue('Total Amount Authorized:'
                        in self.browser.contents)
        self.assertEqual(self.student.current_mode, 'ug_ft')
        self.assertTrue(
            '<input type="hidden" name="amount" value="5175000" />'
            in self.browser.contents)
        self.assertTrue(
            'item_name="School Fee" item_amt="4750000" bank_id="117" '
            'acct_num="1010827641"' in self.browser.contents)


    def test_interswitch_form_pt_acceptance_fees(self):
        self.certificate.study_mode = 'ug_pt'
        self.app['configuration']['2004'].clearance_fee_pt = 43050.0
        # Manager can access InterswitchForm
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('admitted')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['clearance']
        self.browser.getControl("Create ticket").click()
        self.assertTrue('ticket created' in  self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertTrue('Amount Authorized' in self.browser.contents)
        self.assertTrue('<span>43050.0</span>' in  self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="104" />' in
                           self.browser.contents)
        self.assertTrue('Total Amount Authorized:' in self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="4305000" />'
            in self.browser.contents)
        self.assertTrue(
            'item_name="Acceptance Fee" item_amt="4000000" bank_id="117" '
            'acct_num="1014066976"' in self.browser.contents)

    def test_interswitch_form_regular_acceptance_fees(self):
        self.certificate.study_mode = 'ug_ft'
        self.app['configuration']['2004'].clearance_fee = 43050.0
        # Manager can access InterswitchForm
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('admitted')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['clearance']
        self.browser.getControl("Create ticket").click()
        self.assertTrue('ticket created' in self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertTrue('Amount Authorized' in self.browser.contents)
        self.assertTrue('<span>43050.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="102" />'
                        in self.browser.contents)
        self.assertTrue('Total Amount Authorized:' in self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="4305000" />'
            in self.browser.contents)
        self.assertTrue(
            'item_name="Acceptance Fee" item_amt="4280000" bank_id="117" '
            'acct_num="1010827641"' in self.browser.contents)

    def test_interswitch_form_welfare_fees(self):
        self.certificate.study_mode = 'ug_pt'
        self.app['configuration']['2004'].welfare_fee = 1567.0
        # Manager can access InterswitchForm
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('admitted')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['welfare']
        self.browser.getControl("Create ticket").click()
        self.assertTrue('ticket created' in  self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.assertTrue('Amount Authorized' in self.browser.contents)
        self.assertTrue(
            '<span>1567.0</span>' in self.browser.contents)
        self.payment_url = self.browser.url
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('<div class="alert alert-danger">Not applicable</div>'
                        in self.browser.contents)
        self.certificate.study_mode = 'ug_ft'
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="105" />'
                           in self.browser.contents)
        self.assertTrue('Total Amount Authorized:' in self.browser.contents)
        self.assertTrue(
            '<input type="hidden" name="amount" value="156700" />'
            in self.browser.contents)
        self.assertTrue(
            'item_name="AAU Student Welfare Assurance Fee" item_amt="131700" '
            'bank_id="123" acct_num="1006407792"'
            in self.browser.contents)

    def test_interswitch_form_ticket_expired(self):
        self.certificate.study_mode = 'ug_pt'
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        acc_payment = createObject('waeup.StudentOnlinePayment')
        acc_payment.p_state = 'unpaid'
        acc_payment.p_category = 'clearance'
        acc_payment.p_id = 'xyz'
        acc_payment.pay_item_id = '123'
        acc_payment.amount_auth = 876.0
        self.student['payments']['xyz'] = acc_payment
        self.browser.open(self.payments_path + '/xyz')
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('<input type="hidden" name="pay_item_id" value="104" />'
                        in self.browser.contents)
        self.assertTrue('Total Amount Authorized:' in self.browser.contents)
        self.assertEqual(self.student.current_mode, 'ug_pt')
        self.assertTrue(
            '<input type="hidden" name="amount" value="87600" />'
            in self.browser.contents)
        delta = timedelta(days=8)
        acc_payment.creation_date -= delta
        self.browser.open(self.payments_path + '/xyz')
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue(
            'This payment ticket is too old. Please create a new ticket'
            in self.browser.contents)
        delta = timedelta(days=2)
        acc_payment.creation_date += delta
        self.browser.open(self.payments_path + '/xyz')
        self.browser.getLink("CollegePAY", index=0).click()
        self.assertTrue('Total Amount Authorized:' in self.browser.contents)

    @external_test
    def test_webservice(self):
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.payments_path)
        IWorkflowState(self.student).setState('cleared')
        self.student.nationality = u'NG'
        self.browser.open(self.payments_path + '/addop')
        self.browser.getControl(name="form.p_category").value = ['schoolfee']
        self.browser.getControl("Create ticket").click()
        self.assertTrue('ticket created' in self.browser.contents)
        ctrl = self.browser.getControl(name='val_id')
        self.value = ctrl.options[0]
        self.browser.getLink(self.value).click()
        self.payment_url = self.browser.url
        # First we have open InterswitchPageStudent to set provider_amt
        # and gateway_amt
        self.browser.open(self.payment_url + '/goto_interswitch')
        # Now we can call the webservice
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertTrue('Unsuccessful callback' in self.browser.contents)
        # The payment is now in state failed ...
        self.assertTrue('<span>Failed</span>' in self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('failed', 'failed')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'failed')

        # Let's replace the p_id with a valid p_id of the Uniben
        # live system. This is definitely not an appropriate
        # solution for testing, but we have no choice since
        # Interswitch doesn't provide any interface
        # for testing.
        payment = self.student['payments'][self.value]
        payment.p_id = 'p3547789850240'
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertTrue('Callback amount does not match' in
                          self.browser.contents)
        # The payment is now in state failed ...
        self.assertTrue('<span>Failed</span>' in self.browser.contents)
        # Let's replace the amount autorized with the amount of the
        # live system payment
        payment.amount_auth = payment.r_amount_approved
        self.browser.open(self.payment_url + '/request_webservice')
        self.assertTrue('Successful payment' in
                          self.browser.contents)
        # The payment is now in state paid ...
        self.assertTrue('<span>Paid</span>' in self.browser.contents)
        # ... and the catalog has been updated
        cat = getUtility(ICatalog, name='payments_catalog')
        results = list(
            cat.searchResults(p_state=('paid', 'paid')))
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].p_state, 'paid')
        # Approval is logged in students.log ...
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'students.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'zope.mgr - '
            'waeup.aaue.interswitch.browser.CustomInterswitchPaymentRequestWebservicePageStudent - '
            'X1000000 - successful schoolfee payment: p3547789850240\n'
            in logcontent)
        # ... and in payments.log
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'payments.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            '"zope.mgr",X1000000,p3547789850240,schoolfee,'
            '12000.0,00,0.0,150.0,0.0,,,\n'
            in logcontent)
